@@ -8,7 +8,7 @@ use casper_types::{
     account::AccountHash,
     bytesrepr::FromBytes,
     contracts::NamedKeys,
-    system::{auction, handle_payment, mint},
+    system::{auction, handle_payment, mint, CallStackElement},
     BlockTime, CLTyped, CLValue, ContractPackage, DeployHash, EntryPoint, EntryPointType, Key,
     Phase, ProtocolVersion, RuntimeArgs,
 };
@@ -104,6 +104,7 @@ impl Executor {
         protocol_data: ProtocolData,
         system_contract_cache: SystemContractCache,
         contract_package: &ContractPackage,
+        call_stack: Vec<CallStackElement>,
     ) -> ExecutionResult
     where
         R: StateReader<Key, StoredValue>,
@@ -150,7 +151,7 @@ impl Executor {
             access_rights,
             args.clone(),
             authorization_keys,
-            &account,
+            account,
             base_key,
             blocktime,
             deploy_hash,
@@ -166,7 +167,14 @@ impl Executor {
             transfers,
         );
 
-        let mut runtime = Runtime::new(self.config, system_contract_cache, memory, module, context);
+        let mut runtime = Runtime::new(
+            self.config,
+            system_contract_cache,
+            memory,
+            module,
+            context,
+            call_stack,
+        );
 
         let accounts_access_rights = {
             let keys: Vec<Key> = account.named_keys().values().cloned().collect();
@@ -174,11 +182,13 @@ impl Executor {
         };
 
         on_fail_charge!(runtime_context::validate_entry_point_access_with(
-            &contract_package,
+            contract_package,
             entry_point_access,
             |uref| runtime_context::uref_has_access_rights(uref, &accounts_access_rights)
         ));
 
+        let call_stack = runtime.call_stack().to_owned();
+
         if runtime.is_mint(base_key) {
             match runtime.call_host_mint(
                 protocol_version,
@@ -186,6 +196,7 @@ impl Executor {
                 &mut runtime.context().named_keys().to_owned(),
                 &args,
                 Default::default(),
+                call_stack,
             ) {
                 Ok(_value) => {
                     return ExecutionResult::Success {
@@ -210,6 +221,7 @@ impl Executor {
                 &mut runtime.context().named_keys().to_owned(),
                 &args,
                 Default::default(),
+                call_stack,
             ) {
                 Ok(_value) => {
                     return ExecutionResult::Success {
@@ -234,6 +246,7 @@ impl Executor {
                 &mut runtime.context().named_keys().to_owned(),
                 &args,
                 Default::default(),
+                call_stack,
             ) {
                 Ok(_value) => {
                     return ExecutionResult::Success {
@@ -283,6 +296,7 @@ impl Executor {
         phase: Phase,
         protocol_data: ProtocolData,
         system_contract_cache: SystemContractCache,
+        call_stack: Vec<CallStackElement>,
     ) -> ExecutionResult
     where
         R: StateReader<Key, StoredValue>,
@@ -309,7 +323,7 @@ impl Executor {
             payment_named_keys,
             Default::default(),
             payment_base_key,
-            &account,
+            account,
             authorization_keys,
             blocktime,
             deploy_hash,
@@ -323,6 +337,7 @@ impl Executor {
             phase,
             protocol_data,
             system_contract_cache,
+            call_stack,
         ) {
             Ok((_instance, runtime)) => runtime,
             Err(error) => {
@@ -371,6 +386,7 @@ impl Executor {
         phase: Phase,
         protocol_data: ProtocolData,
         system_contract_cache: SystemContractCache,
+        call_stack: Vec<CallStackElement>,
     ) -> (Option<T>, ExecutionResult)
     where
         R: StateReader<Key, StoredValue>,
@@ -456,6 +472,7 @@ impl Executor {
             phase,
             protocol_data,
             system_contract_cache,
+            call_stack,
         ) {
             Ok((instance, runtime)) => (instance, runtime),
             Err(error) => {
@@ -503,6 +520,7 @@ impl Executor {
         phase: Phase,
         protocol_data: ProtocolData,
         system_contract_cache: SystemContractCache,
+        call_stack: Vec<CallStackElement>,
     ) -> Result<T, Error>
     where
         R: StateReader<Key, StoredValue>,
@@ -533,6 +551,7 @@ impl Executor {
             phase,
             protocol_data,
             system_contract_cache,
+            call_stack,
         )?;
 
         let error: wasmi::Error = match instance.invoke_export(entry_point_name, &[], &mut runtime)
@@ -590,6 +609,7 @@ impl Executor {
         phase: Phase,
         protocol_data: ProtocolData,
         system_contract_cache: SystemContractCache,
+        call_stack: Vec<CallStackElement>,
     ) -> Result<(ModuleRef, Runtime<'a, R>), Error>
     where
         R: StateReader<Key, StoredValue>,
@@ -639,6 +659,7 @@ impl Executor {
             memory,
             module,
             runtime_context,
+            call_stack,
         );
 
         Ok((instance, runtime))
@@ -685,6 +706,9 @@ impl DirectSystemContractCall {
         T: FromBytes + CLTyped,
     {
         let entry_point_name = self.entry_point_name();
+
+        let call_stack = runtime.call_stack().to_owned();
+
         let result = match self {
             DirectSystemContractCall::Slash
             | DirectSystemContractCall::RunAuction
@@ -694,6 +718,7 @@ impl DirectSystemContractCall {
                 named_keys,
                 runtime_args,
                 extra_keys,
+                call_stack,
             ),
             DirectSystemContractCall::FinalizePayment => runtime.call_host_handle_payment(
                 protocol_version,
@@ -701,6 +726,7 @@ impl DirectSystemContractCall {
                 named_keys,
                 runtime_args,
                 extra_keys,
+                call_stack,
             ),
             DirectSystemContractCall::CreatePurse | DirectSystemContractCall::Transfer => runtime
                 .call_host_mint(
@@ -709,6 +735,7 @@ impl DirectSystemContractCall {
                     named_keys,
                     runtime_args,
                     extra_keys,
+                    call_stack,
                 ),
             DirectSystemContractCall::GetEraValidators => runtime.call_host_auction(
                 protocol_version,
@@ -716,6 +743,7 @@ impl DirectSystemContractCall {
                 named_keys,
                 runtime_args,
                 extra_keys,
+                call_stack,
             ),
 
             DirectSystemContractCall::GetPaymentPurse => runtime.call_host_handle_payment(
@@ -724,6 +752,7 @@ impl DirectSystemContractCall {
                 named_keys,
                 runtime_args,
                 extra_keys,
+                call_stack,
             ),
         };
 
