@@ -17,13 +17,15 @@ use casper_types::{
     system::auction::EraInfo,
     AccessRights, BlockTime, CLType, CLValue, Contract, ContractPackage, ContractPackageHash,
     DeployHash, DeployInfo, EntryPointAccess, EntryPointType, Key, KeyTag, Phase, ProtocolVersion,
-    PublicKey, RuntimeArgs, Transfer, TransferAddr, URef, KEY_HASH_LENGTH,
+    PublicKey, RuntimeArgs, Transfer, TransferAddr, URef, DICTIONARY_ITEM_KEY_MAX_LENGTH,
+    KEY_HASH_LENGTH,
 };
 
 use crate::{
     core::{
         engine_state::execution_effect::ExecutionEffect,
         execution::{AddressGenerator, Error},
+        runtime_context::dictionary::DictionaryValue,
         tracking_copy::{AddResult, TrackingCopy},
         Address,
     },
@@ -31,6 +33,7 @@ use crate::{
     storage::{global_state::StateReader, protocol_data::ProtocolData},
 };
 
+pub(crate) mod dictionary;
 #[cfg(test)]
 mod tests;
 
@@ -168,7 +171,7 @@ where
     }
 
     pub fn named_keys(&self) -> &NamedKeys {
-        &self.named_keys
+        self.named_keys
     }
 
     pub fn named_keys_mut(&mut self) -> &mut NamedKeys {
@@ -260,6 +263,10 @@ where
                 self.named_keys.remove(name);
                 Ok(())
             }
+            Key::Dictionary(_) => {
+                self.named_keys.remove(name);
+                Ok(())
+            }
         }
     }
 
@@ -284,7 +291,7 @@ where
     }
 
     pub fn account(&self) -> &'a Account {
-        &self.account
+        self.account
     }
 
     pub fn args(&self) -> &RuntimeArgs {
@@ -417,7 +424,7 @@ where
         T: TryFrom<StoredValue>,
         T::Error: Debug,
     {
-        let value = match self.read_gs(&key)? {
+        let value = match self.read_gs(key)? {
             None => return Err(Error::KeyNotFound(*key)),
             Some(value) => value,
         };
@@ -494,7 +501,7 @@ where
     ) -> Result<[u8; KEY_HASH_LENGTH], Error> {
         let new_hash = self.new_hash_address()?;
         self.validate_value(&contract)?;
-        self.metered_write_gs_unsafe(new_hash, contract)?;
+        self.metered_write_gs_unsafe(Key::Hash(new_hash), contract)?;
         Ok(new_hash)
     }
 
@@ -526,44 +533,48 @@ where
         &mut self.transfers
     }
 
+    fn validate_cl_value(&self, cl_value: &CLValue) -> Result<(), Error> {
+        match cl_value.cl_type() {
+            CLType::Bool
+            | CLType::I32
+            | CLType::I64
+            | CLType::U8
+            | CLType::U32
+            | CLType::U64
+            | CLType::U128
+            | CLType::U256
+            | CLType::U512
+            | CLType::Unit
+            | CLType::String
+            | CLType::Option(_)
+            | CLType::List(_)
+            | CLType::ByteArray(..)
+            | CLType::Result { .. }
+            | CLType::Map { .. }
+            | CLType::Tuple1(_)
+            | CLType::Tuple3(_)
+            | CLType::Any
+            | CLType::PublicKey => Ok(()),
+            CLType::Key => {
+                let key: Key = cl_value.to_owned().into_t()?; // TODO: optimize?
+                self.validate_key(&key)
+            }
+            CLType::URef => {
+                let uref: URef = cl_value.to_owned().into_t()?; // TODO: optimize?
+                self.validate_uref(&uref)
+            }
+            tuple @ CLType::Tuple2(_) if *tuple == casper_types::named_key_type() => {
+                let (_name, key): (String, Key) = cl_value.to_owned().into_t()?; // TODO: optimize?
+                self.validate_key(&key)
+            }
+            CLType::Tuple2(_) => Ok(()),
+        }
+    }
+
     /// Validates whether keys used in the `value` are not forged.
     fn validate_value(&self, value: &StoredValue) -> Result<(), Error> {
         match value {
-            StoredValue::CLValue(cl_value) => match cl_value.cl_type() {
-                CLType::Bool
-                | CLType::I32
-                | CLType::I64
-                | CLType::U8
-                | CLType::U32
-                | CLType::U64
-                | CLType::U128
-                | CLType::U256
-                | CLType::U512
-                | CLType::Unit
-                | CLType::String
-                | CLType::Option(_)
-                | CLType::List(_)
-                | CLType::ByteArray(..)
-                | CLType::Result { .. }
-                | CLType::Map { .. }
-                | CLType::Tuple1(_)
-                | CLType::Tuple3(_)
-                | CLType::Any
-                | CLType::PublicKey => Ok(()),
-                CLType::Key => {
-                    let key: Key = cl_value.to_owned().into_t()?; // TODO: optimize?
-                    self.validate_key(&key)
-                }
-                CLType::URef => {
-                    let uref: URef = cl_value.to_owned().into_t()?; // TODO: optimize?
-                    self.validate_uref(&uref)
-                }
-                tuple @ CLType::Tuple2(_) if *tuple == casper_types::named_key_type() => {
-                    let (_name, key): (String, Key) = cl_value.to_owned().into_t()?; // TODO: optimize?
-                    self.validate_key(&key)
-                }
-                CLType::Tuple2(_) => Ok(()),
-            },
+            StoredValue::CLValue(cl_value) => self.validate_cl_value(cl_value),
             StoredValue::Account(account) => {
                 // This should never happen as accounts can't be created by contracts.
                 // I am putting this here for the sake of completeness.
@@ -632,7 +643,7 @@ where
     }
 
     fn validate_readable(&self, key: &Key) -> Result<(), Error> {
-        if self.is_readable(&key) {
+        if self.is_readable(key) {
             Ok(())
         } else {
             Err(Error::InvalidAccess {
@@ -642,7 +653,7 @@ where
     }
 
     fn validate_addable(&self, key: &Key) -> Result<(), Error> {
-        if self.is_addable(&key) {
+        if self.is_addable(key) {
             Ok(())
         } else {
             Err(Error::InvalidAccess {
@@ -652,7 +663,7 @@ where
     }
 
     fn validate_writeable(&self, key: &Key) -> Result<(), Error> {
-        if self.is_writeable(&key) {
+        if self.is_writeable(key) {
             Ok(())
         } else {
             Err(Error::InvalidAccess {
@@ -673,6 +684,11 @@ where
             Key::Balance(_) => false,
             Key::Bid(_) => true,
             Key::Withdraw(_) => true,
+            Key::Dictionary(_) => {
+                // Dictionary is a special case that will not be readable by default, but the access
+                // bits are verified from within API call.
+                false
+            }
         }
     }
 
@@ -687,6 +703,11 @@ where
             Key::Balance(_) => false,
             Key::Bid(_) => false,
             Key::Withdraw(_) => false,
+            Key::Dictionary(_) => {
+                // Dictionary is a special case that will not be readable by default, but the access
+                // bits are verified from within API call.
+                false
+            }
         }
     }
 
@@ -701,6 +722,11 @@ where
             Key::Balance(_) => false,
             Key::Bid(_) => false,
             Key::Withdraw(_) => false,
+            Key::Dictionary(_) => {
+                // Dictionary is a special case that will not be readable by default, but the access
+                // bits are verified from within API call.
+                false
+            }
         }
     }
 
@@ -1028,4 +1054,68 @@ where
         self.validate_uref(&contract_package.access_key())?;
         Ok(contract_package)
     }
+
+    pub(crate) fn dictionary_get(
+        &mut self,
+        uref: URef,
+        dictionary_item_key: &str,
+    ) -> Result<Option<CLValue>, Error> {
+        self.validate_readable(&uref.into())?;
+        self.validate_key(&uref.into())?;
+        let dictionary_item_key_bytes = dictionary_item_key.as_bytes();
+
+        if dictionary_item_key_bytes.len() > DICTIONARY_ITEM_KEY_MAX_LENGTH {
+            return Err(Error::DictionaryItemKeyExceedsLength);
+        }
+
+        let dictionary_key = Key::dictionary(uref, dictionary_item_key_bytes);
+
+        let maybe_stored_value = self
+            .tracking_copy
+            .borrow_mut()
+            .read(self.correlation_id, &dictionary_key)
+            .map_err(Into::into)?;
+
+        if let Some(stored_value) = maybe_stored_value {
+            let cl_value_indirect: CLValue =
+                stored_value.try_into().map_err(Error::TypeMismatch)?;
+            let dictionary_value: DictionaryValue =
+                cl_value_indirect.into_t().map_err(Error::from)?;
+            let cl_value = dictionary_value.into_cl_value();
+            Ok(Some(cl_value))
+        } else {
+            Ok(None)
+        }
+    }
+
+    pub fn dictionary_put(
+        &mut self,
+        seed_uref: URef,
+        dictionary_item_key: &str,
+        cl_value: CLValue,
+    ) -> Result<(), Error> {
+        let dictionary_item_key_bytes = dictionary_item_key.as_bytes();
+
+        if dictionary_item_key_bytes.len() > DICTIONARY_ITEM_KEY_MAX_LENGTH {
+            return Err(Error::DictionaryItemKeyExceedsLength);
+        }
+
+        self.validate_writeable(&seed_uref.into())?;
+        self.validate_uref(&seed_uref)?;
+
+        self.validate_cl_value(&cl_value)?;
+
+        let wrapped_cl_value = {
+            let dictionary_value = DictionaryValue::new(
+                cl_value,
+                seed_uref.addr().to_vec(),
+                dictionary_item_key_bytes.to_vec(),
+            );
+            CLValue::from_t(dictionary_value).map_err(Error::from)?
+        };
+
+        let dictionary_key = Key::dictionary(seed_uref, dictionary_item_key_bytes);
+        self.metered_write_gs_unsafe(dictionary_key, wrapped_cl_value)?;
+        Ok(())
+    }
 }
