@@ -18,7 +18,10 @@ use casper_types::{bytesrepr, CLType, CLValue, CLValueError, Key, KeyTag, Tagged
 pub use self::ext::TrackingCopyExt;
 use self::meter::{heap_meter::HeapSize, Meter};
 use crate::{
-    core::engine_state::{execution_effect::ExecutionEffect, op::Op},
+    core::{
+        engine_state::{execution_effect::ExecutionEffect, op::Op},
+        runtime_context::dictionary,
+    },
     shared::{
         additive_map::AdditiveMap,
         newtypes::{Blake2bHash, CorrelationId},
@@ -190,7 +193,7 @@ impl<M: Meter<Key, StoredValue>> TrackingCopyCache<M> {
 
     /// Gets value from `key` in the cache.
     pub fn get(&mut self, key: &Key) -> Option<&StoredValue> {
-        if let Some(value) = self.muts_cached.get(&key) {
+        if let Some(value) = self.muts_cached.get(key) {
             return Some(value);
         };
 
@@ -289,7 +292,7 @@ impl<R: StateReader<Key, StoredValue>> TrackingCopy<R> {
         key_tag: &KeyTag,
     ) -> Result<BTreeSet<Key>, R::Error> {
         let mut ret: BTreeSet<Key> = BTreeSet::new();
-        match self.cache.get_key_tag_reads_cached(&key_tag) {
+        match self.cache.get_key_tag_reads_cached(key_tag) {
             Some(keys) => ret.extend(keys),
             None => {
                 let key_tag = key_tag.to_owned();
@@ -300,7 +303,7 @@ impl<R: StateReader<Key, StoredValue>> TrackingCopy<R> {
                 self.cache.insert_key_tag_read(key_tag, ret.to_owned())
             }
         }
-        if let Some(keys) = self.cache.get_key_tag_muts_cached(&key_tag) {
+        if let Some(keys) = self.cache.get_key_tag_muts_cached(key_tag) {
             ret.extend(keys)
         }
         Ok(ret)
@@ -448,6 +451,18 @@ impl<R: StateReader<Key, StoredValue>> TrackingCopy<R> {
 
             let value = stored_value.value().to_owned();
 
+            // Following code does a patching on the `StoredValue` that unwraps an inner
+            // `DictionaryValue` for dictionaries only.
+            let value = match dictionary::handle_stored_value(query.current_key, value) {
+                Ok(patched_stored_value) => patched_stored_value,
+                Err(error) => {
+                    return Ok(query.into_not_found_result(&format!(
+                        "Failed to retrieve dictionary value: {}",
+                        error
+                    )))
+                }
+            };
+
             proofs.push(stored_value);
 
             if query.unvisited_names.is_empty() {
@@ -494,25 +509,25 @@ impl<R: StateReader<Key, StoredValue>> TrackingCopy<R> {
                     }
                 }
                 StoredValue::ContractPackage(_) => {
-                    return Ok(query.into_not_found_result(&"ContractPackage value found."));
+                    return Ok(query.into_not_found_result("ContractPackage value found."));
                 }
                 StoredValue::ContractWasm(_) => {
-                    return Ok(query.into_not_found_result(&"ContractWasm value found."));
+                    return Ok(query.into_not_found_result("ContractWasm value found."));
                 }
                 StoredValue::Transfer(_) => {
-                    return Ok(query.into_not_found_result(&"Transfer value found."));
+                    return Ok(query.into_not_found_result("Transfer value found."));
                 }
                 StoredValue::DeployInfo(_) => {
-                    return Ok(query.into_not_found_result(&"DeployInfo value found."));
+                    return Ok(query.into_not_found_result("DeployInfo value found."));
                 }
                 StoredValue::EraInfo(_) => {
-                    return Ok(query.into_not_found_result(&"EraInfo value found."));
+                    return Ok(query.into_not_found_result("EraInfo value found."));
                 }
                 StoredValue::Bid(_) => {
-                    return Ok(query.into_not_found_result(&"Bid value found."));
+                    return Ok(query.into_not_found_result("Bid value found."));
                 }
                 StoredValue::Withdraw(_) => {
-                    return Ok(query.into_not_found_result(&"UnbondingPurses value found."));
+                    return Ok(query.into_not_found_result("UnbondingPurses value found."));
                 }
             }
         }
