@@ -17,6 +17,7 @@ use super::{
 use crate::{
     core::{
         engine_state::{op::Op, EngineConfig},
+        runtime_context::dictionary,
         ValidationError,
     },
     shared::{
@@ -91,8 +92,8 @@ fn tracking_copy_new() {
     let db = CountingDb::new(counter);
     let tc = TrackingCopy::new(db);
 
-    assert_eq!(tc.ops.is_empty(), true);
-    assert_eq!(tc.fns.is_empty(), true);
+    assert!(tc.ops.is_empty());
+    assert!(tc.fns.is_empty());
 }
 
 #[test]
@@ -231,8 +232,8 @@ fn tracking_copy_add_named_key() {
         StoredValue::CLValue(CLValue::from_t(3_i32).unwrap()),
     );
     assert_matches!(failed_add, Ok(AddResult::TypeMismatch(_)));
-    assert_eq!(tc.ops.is_empty(), true);
-    assert_eq!(tc.fns.is_empty(), true);
+    assert!(tc.ops.is_empty());
+    assert!(tc.fns.is_empty());
 
     // adding correct type works
     let add = tc.add(correlation_id, k, named_key);
@@ -314,7 +315,10 @@ proptest! {
     #[test]
     fn query_empty_path(k in key_arb(), missing_key in key_arb(), v in stored_value_arb()) {
         let correlation_id = CorrelationId::new();
-        let (gs, root_hash) = InMemoryGlobalState::from_pairs(correlation_id, &[(k, v.to_owned())]).unwrap();
+
+        let value = dictionary::handle_stored_value_into(k, v.clone()).unwrap();
+
+        let (gs, root_hash) = InMemoryGlobalState::from_pairs(correlation_id, &[(k, value)]).unwrap();
         let view = gs.checkout(root_hash).unwrap().unwrap();
         let tc = TrackingCopy::new(view);
         let empty_path = Vec::new();
@@ -351,9 +355,11 @@ proptest! {
         ));
         let contract_key = Key::Hash(hash);
 
+        let value = dictionary::handle_stored_value_into(k, v.clone()).unwrap();
+
         let (gs, root_hash) = InMemoryGlobalState::from_pairs(
             correlation_id,
-            &[(k, v.to_owned()), (contract_key, contract)]
+            &[(k, value), (contract_key, contract)]
         ).unwrap();
         let view = gs.checkout(root_hash).unwrap().unwrap();
         let tc = TrackingCopy::new(view);
@@ -370,7 +376,6 @@ proptest! {
         }
     }
 
-
     #[test]
     fn query_account_state(
         k in key_arb(), // key state is stored at
@@ -393,9 +398,11 @@ proptest! {
         );
         let account_key = Key::Account(address);
 
+        let value = dictionary::handle_stored_value_into(k, v.clone()).unwrap();
+
         let (gs, root_hash) = InMemoryGlobalState::from_pairs(
             correlation_id,
-            &[(k, v.to_owned()), (account_key, StoredValue::Account(account))],
+            &[(k, value), (account_key, StoredValue::Account(account))],
         ).unwrap();
         let view = gs.checkout(root_hash).unwrap().unwrap();
         let tc = TrackingCopy::new(view);
@@ -450,8 +457,10 @@ proptest! {
         );
         let account_key = Key::Account(address);
 
+        let value = dictionary::handle_stored_value_into(k, v.clone()).unwrap();
+
         let (gs, root_hash) = InMemoryGlobalState::from_pairs(correlation_id, &[
-            (k, v.to_owned()),
+            (k, value),
             (contract_key, contract),
             (account_key, StoredValue::Account(account)),
         ]).unwrap();
